<?php
/**
 * カテゴリ別記事一覧 表示用カスタムウィジェット
 *
 * カテゴリで階層化された記事の一覧を表示する
 *
 * @link http://codex.wordpress.org/Widgets_API#Developing_Widgets
 *
 * @package     WordPress
 * @subpackage  SmartStyle
 * @since       
 */
class SmartWidget_CategoryPostList extends WP_Widget {
	const CHAR_INDENT = "\t";	// インデント
	const CHAR_CR = "\n";		// 改行

	/**
	 * コンストラクタ
	 *   widget actual processes
	 * 
	 * @param  
	 * @return PostList
	 */
	function __construct() {
		$name = 'カテゴリー別記事一覧';
		$widget_ops = array('classname' => 'smartwidget_categorypostlist', 'description' => 'カテゴリで階層化された記事の一覧を表示します。');
		parent::__construct('categorypostlist', $name, $widget_ops);
	}

	/**
	 * 管理画面のオプション設定フォーム出力
	 *   outputs the options form on admin
	 * 
	 * @param  array $instance ウィジェット設定値
	 */
	function form($instance) {
		$instance = wp_parse_args((array)$instance, array('title'=>'', 'csort'=>'name', 'corder'=>'asc', 'psort'=>'ID', 'porder'=>'asc'));
		$title   = esc_attr($instance['title']);										// タイトル
		$isclose = isset($instance['isclose']) ? (bool)$instance['isclose'] : false;	// 開閉型ツリーにするか否か
		$indent = $this->get_indent(2);		// 出力インデント

		$output = $indent . '<p><label for="' . esc_attr($this->get_field_id('title')) . '">タイトル</label>' . self::CHAR_CR
				. $indent . '<input type="text" class="widefat" id="' . esc_attr($this->get_field_id('title')) . '"'
						  . ' name="' . esc_attr($this->get_field_name('title')) . '" value="' . esc_attr($title) . '"></p>' . self::CHAR_CR

				. $indent . '<p><label for="' . esc_attr($this->get_field_id('csort')) . '">カテゴリ並び順</label><br />' . self::CHAR_CR
				. $indent . '<select name="' . esc_attr($this->get_field_name('csort')) . '" id="' . esc_attr($this->get_field_id('csort')) . '">' . self::CHAR_CR
				. $indent . '<option value="id"'    . (($instance['csort'] == 'id')    ? ' selected="selected"' : '') . '>ID</option>'       . self::CHAR_CR
				. $indent . '<option value="name"'  . (($instance['csort'] == 'name')  ? ' selected="selected"' : '') . '>名前</option>'     . self::CHAR_CR
				. $indent . '<option value="slug"'  . (($instance['csort'] == 'slug')  ? ' selected="selected"' : '') . '>スラッグ</option>' . self::CHAR_CR
				. $indent . '<option value="count"' . (($instance['csort'] == 'count') ? ' selected="selected"' : '') . '>記事数</option>'   . self::CHAR_CR
				. $indent . '<option value="group"' . (($instance['csort'] == 'group') ? ' selected="selected"' : '') . '>グループ</option>' . self::CHAR_CR
				. $indent . '</select>' . self::CHAR_CR
				. $indent . '<select name="' . esc_attr($this->get_field_name('corder')) . '" id="' . esc_attr($this->get_field_id('corder')) . '">' . self::CHAR_CR
				. $indent . '<option value="asc"'  . (($instance['corder'] == 'asc')   ? ' selected="selected"' : '') . '>昇順</option>' . self::CHAR_CR
				. $indent . '<option value="desc"' . (($instance['corder'] == 'desc')  ? ' selected="selected"' : '') . '>降順</option>' . self::CHAR_CR
				. $indent . '</select>' . self::CHAR_CR

				. $indent . '<p><label for="' . esc_attr($this->get_field_id('psort')) . '">記事並び順</label><br />' . self::CHAR_CR
				. $indent . '<select name="' . esc_attr($this->get_field_name('psort')) . '" id="' . esc_attr($this->get_field_id('psort')) . '">' . self::CHAR_CR
				. $indent . '<option value="ID"'       . (($instance['psort'] == 'ID')       ? ' selected="selected"' : '') . '>ID</option>'       . self::CHAR_CR
				. $indent . '<option value="author"'   . (($instance['psort'] == 'author')   ? ' selected="selected"' : '') . '>投稿者</option>'   . self::CHAR_CR
				. $indent . '<option value="title"'    . (($instance['psort'] == 'title')    ? ' selected="selected"' : '') . '>タイトル</option>' . self::CHAR_CR
				. $indent . '<option value="date"'     . (($instance['psort'] == 'date')     ? ' selected="selected"' : '') . '>投稿日時</option>' . self::CHAR_CR
				. $indent . '<option value="modified"' . (($instance['psort'] == 'modified') ? ' selected="selected"' : '') . '>更新日時</option>' . self::CHAR_CR
				. $indent . '</select>' . self::CHAR_CR
				. $indent . '<select name="' . esc_attr($this->get_field_name('porder')) . '" id="' . esc_attr($this->get_field_id('porder')) . '">' . self::CHAR_CR
				. $indent . '<option value="asc"'  . (($instance['porder'] == 'asc')   ? ' selected="selected"' : '') . '>昇順</option>' . self::CHAR_CR
				. $indent . '<option value="desc"' . (($instance['porder'] == 'desc')  ? ' selected="selected"' : '') . '>降順</option>' . self::CHAR_CR
				. $indent . '</select>' . self::CHAR_CR

				. $indent . '<p><label for="' . esc_attr($this->get_field_id('isclose')) . '">開閉式ツリーにしますか？</label>' . self::CHAR_CR
				. $indent . '<input type="checkbox" class="checkbox" id="' . esc_attr($this->get_field_id('isclose')) . '"'
						  . ' name="' . esc_attr($this->get_field_name('isclose')) . '"'
						  . ' value="1"' . (($isclose) ? ' checked="checked"' : '') . '></p>' . self::CHAR_CR;

		echo $output;
	}

	/**
	 * 管理画面のオプション設定値の保存
	 *   processes widget options to be saved
	 *
	 * @param  array $new_instance  入力ウィジェット設定値
	 * @param  array $old_instance  既存ウィジェット設定値
	 * @return array 更新済ウィジェット設定値
	 */
	function update($new_instance, $old_instance) {
		$instance = $old_instance;

		$instance['title'] = strip_tags($new_instance['title']);

		if (in_array($new_instance['csort'], array('id', 'name', 'slug', 'count', 'group'))) {
			$instance['csort'] = $new_instance['csort'];
		} else {
			$instance['csort'] = 'name';
		}
		if (in_array($new_instance['corder'], array('asc', 'desc'))) {
			$instance['corder'] = $new_instance['corder'];
		} else {
			$instance['corder'] = 'asc';
		}

		if (in_array($new_instance['psort'], array('ID', 'author', 'title', 'date', 'modified'))) {
			$instance['psort'] = $new_instance['psort'];
		} else {
			$instance['psort'] = 'ID';
		}
		if (in_array($new_instance['porder'], array('asc', 'desc'))) {
			$instance['porder'] = $new_instance['porder'];
		} else {
			$instance['porder'] = 'asc';
		}

		$instance['isclose'] = isset($new_instance['isclose']) ? 1 : 0;

		return $instance;
	}

	/**
	 * ウィジェットHTML出力
	 *   outputs the content of the widget
	 *
	 * @param  array $args     ウイジェット配置用サイドバーパラメータ(register_sidebarで)
	 * @param  array $instance ウィジェット設定値
	 */
	function widget($args, $instance) {
		extract($args);																	// サイドバーパラメータ配列のシンボル変数展開
		$title   = empty($instance['title'])   ? '' : esc_attr($instance['title']);		// タイトル
		$csort   = isset($instance['csort'])  ? $instance['csort']  : 'name';			// カテゴリソート対象
		$corder  = isset($instance['corder']) ? $instance['corder'] : 'asc';			// カテゴリソート順
		$psort   = isset($instance['psort'])  ? $instance['psort']  : 'id';				// 記事ソート対象
		$porder  = isset($instance['porder']) ? $instance['porder'] : 'asc';			// 記事ソート順
		$isclose = empty($instance['isclose']) ?  false : (bool)$instance['isclose'];	// 開閉型ツリーにするか否か
		$level = 1;		// 開始階層(インデント数)指定

		// ウィジェット出力開始
		echo $before_widget . self::CHAR_CR;

		// インデント設定
		$indent = $this->get_indent($level);

		// タイトルの指定がある場合はタイトル出力
		$title = trim($title);
		if ($title != '') {
			echo $indent . $before_title . $title . $after_title . self::CHAR_CR;
		}

		// リスト出力開始
		echo $indent . '<ul>' . self::CHAR_CR;

		// カテゴリ取得
		/*
		//-- 以下は取得パラメータデフォルト値
		$args = array(
			'type'         => 'post',	// 取得するカテゴリの種別(post:初期値, link)
			'child_of'     => '',		// カテゴリIDで指定されたカテゴリの子孫カテゴリを取得(このパラメータが指定された場合、hide_emptyパラメータは自動的にfalseになる)
			'parent'       => '',		// カテゴリIDで指定されたカテゴリの直接の子カテゴリのみを取得
			'orderby'      => 'name',	// ソート対象(id, name:初期値, slug, count, group)
			'order'        => 'asc',	// ソート順(asc[昇順]:初期値, desc[降順])
			'hide_empty'   => 1,		// 記事がないカテゴリを排除するか否か(1[true]:初期値, 0[false])
			'hierarchical' => 1,		// 記事がない空の子カテゴリでも、さらにその子カテゴリが空でなければ問い合わせ結果に含まれるよう階層的に判断するか否か(1[true]:初期値, 0[false])
			'exclude'      => '',		// 除外したいカテゴリID(複数指定する場合は,で区切る)
			'include'      => '',		// 取得したいカテゴリID(複数指定する場合は,で区切る)
			'number'       => '',		// 取得件数(省略時は全件)
			'pad_counts'   => 0			// 子カテゴリを含め、リンクまたは記事の数を計算するか否か(1[true], 0[false]:初期値)
		);
		//-- 以下は戻り値配列内容([*]は /wp-includes/category.php を読んだ限りではあるかどうか怪しいもの)
			term_id              = int    : ID
			name                 = string : 名前
			slug                 = string : スラッグ
			[*]term_group        = int    : グループID
			[*]term_taxonomy_id  = int    : タクソノミーID
			[*]taxonomy          = string : タクソノミー名、カテゴリーの場合は必ず'category'となる
			description          = string : 説明
			parent               = int    : 親カテゴリーID、親カテゴリーがない場合は0となる
			count                = int    : 投稿数
			cat_ID               = int    : カテゴリーID（term_idのエイリアス）
			category_count       = int    : 投稿数（countのエイリアス）
			category_description = string : 説明（descriptionのエイリアス）
			cat_name             = string : カテゴリー名（nameのエイリアス）
			category_nicename    = string : ナイスネーム（slugのエイリアス）
			category_parent      = int    : 親カテゴリーID（parentのエイリアス）
		*/
		//$lcategories = get_categories("orderby=id&order=asc&hide_empty=1&hierarchical=1&pad_counts=1");
		$args = array(
			'orderby'      => $csort,
			'order'        => $corder,
			'hide_empty'   => 1,
			'hierarchical' => 1,
			'pad_counts'   => 1
		);
		$lcategories = get_categories(http_build_query($args));

		// 親カテゴリを指定して、再帰的に子カテゴリ(及び直属の投稿記事)を出力
		$this->outputCategory(0, $level, $lcategories, $isclose, $psort, $porder);

		// リスト出力終了
		echo $indent . '</ul>' . self::CHAR_CR;

		// ウィジェット出力終了
		echo $after_widget . self::CHAR_CR;
	}

	/**
	 * 親カテゴリを指定しての下層カテゴリのHTML出力
	 *
	 * @access private
	 * @param  int    $parentID   親カテゴリID
	 * @param  int    $level      階層
	 * @param  array  $categories カテゴリオブジェクト配列(ソート順を指定して全件取得済であること)
	 * @param  bool   $isclose    開閉型ツリーにするか否か
	 * @param  string $psort      記事ソート対象
	 * @param  string $porder     記事ソート順
	 */
	function outputCategory($parentID, $level, $categories, $isclose, $psort, $porder) {
		$urlInfo = parse_url(home_url());		// JavaScriptに渡すcookie書込み用path

		// 階層のインクリメント
		$level += 1;

		// 指定された親カテゴリでフィルタ
		$filter = create_function('$categories', 'return ($categories->category_parent == ' . $parentID . ');');
		$lcategories = array_filter($categories, $filter);

		// インデント設定
		$indent = $this->get_indent($level);

		// フィルタ済カテゴリ全件処理
		foreach ($lcategories as $lcategory) {
			// カテゴリリンクの出力
			echo $indent;
			echo '<li class="list_category" id="swparent_' . $lcategory->term_id . '"' . (($isclose) ? ' name="swparents"' : '') . '>';
			if ($isclose) echo '<a class="tree" href="javascript:void(0);" onClick="SmartWidgetTree(\'' . $urlInfo["path"] . '\',\'' . $lcategory->term_id . '\'); return false;">';
			echo '<img id="swfolder_' . $lcategory->term_id . '" src="' . get_template_directory_uri() . '/images/' . (($isclose) ? 'tree_' : '') .  'folder_open.gif" alt="カテゴリ" />';
			if ($isclose) echo '</a>';
			echo '<h3 class="category_title">';
			echo '<a href="' . get_category_link($lcategory->term_id) . '" title="' . $lcategory->name . '">' . $lcategory->name . '</a>(' . $lcategory->count . ')';
			echo '</h3>';
			echo '</li>' . self::CHAR_CR;

			// 下層リスト出力開始
			echo $indent . '<li class="list_children" id="swchild_' . $lcategory->term_id . '"><ul>' . self::CHAR_CR;
			// 親カテゴリを指定して、再帰的に子カテゴリ(及び直属の投稿記事)を出力
			$this->outputCategory($lcategory->term_id, $level, $categories, $isclose, $psort, $porder);
			// 下層リスト出力終了
			echo $indent . '</ul></li>' . self::CHAR_CR;
		}

		// 配下(直属)の投稿記事を出力
		$this->outputPosts($parentID, $level, $psort, $porder);
	}

	/**
	 * 親カテゴリを指定しての投稿記事のHTML出力
	 *
	 * @access private
	 * @param  int    $parentCategory 親カテゴリID
	 * @param  int    $level          階層
	 * @param  string $psort          記事ソート対象
	 * @param  string $porder         記事ソート順
	 */
	function outputPosts($parentCategory, $level, $psort, $porder) {

		// 投稿記事取得
		/*
		//-- 以下は取得パラメータデフォルト値
		$args = array(
			'numberposts'      => 5,				// 取得件数(省略時は5)、0で管理画面にて設定する「1ページに表示する最大投稿数」、0以下で全件、posts_per_pageと互換と謳っているがウソ
			'offset'           => 0,				// 取得開始位置(省略時は0[先頭])
			'category'         => '',				// 所属カテゴリID(複数指定する場合は,で区切る)
			'orderby'          => 'post_date',		// ソート対象(none, ID, author, title, date:初期値, modified, parent, rand, comment_count, menu_order, meta_value, meta_value_num)
			'order'            => 'desc',			// ソート順(asc[昇順], desc[降順]:初期値)
			'include'          => '',				// 取得したい投稿ID(複数指定する場合は,で区切る)
			'exclude'          => '',				// 除外したい投稿ID(複数指定する場合は,で区切る)
			'meta_key'         => '',				// カスタムフィールドの名前
			'meta_value'       => '',				// カスタムフィールドの値
			'post_type'        => 'post',			// 投稿情報タイプ(post[投稿ページ]:初期値, page[固定ページ], attachment[添付], any）
			'post_mime_type'   => '',				// 
			'post_parent'      => '',				// 親投稿ID
			'post_status'      => 'publish',		// 投稿ステータス(auto-draft, draft, inherit:post_typeattachmentの場合の初期値, private, publish:初期値, any)
			'suppress_filters' => true				// 最終的な検索条件に対してフィルター処理を行わないか否か(1[true]:初期値, 0[false])
		);
		//-- 以下は戻り値配列内容([*]は /wp-includes/post.php を読んだ限りではあるかどうか怪しいもの)
			ID                    = int    : 投稿ID
			post_author           = string : 投稿者ID
			post_date             = string : 投稿日時
			post_date_gmt         = string : 投稿日時（GMT）
			post_content          = string : 投稿内容
			post_title            = string : 投稿タイトル
			post_excerpt          = string : 投稿内容（抜粋）
			post_status           = string : 投稿状態（'publish'、'private'など）
			comment_status        = string : コメント投稿状態（'open'など）
			ping_status           = string : トラックバック/ピンバック受付状態（'open'など）
			post_password         = string : パスワード（公開状態がパスワード保護時）
			post_name             = string : 投稿名（パーマリンク設定で%postname%使用時）
			to_ping               = string : 
			pinged                = string : 
			post_modified         = string : 更新日時
			post_modified_gmt     = string : 更新日時（GMT）
			post_content_filtered = string : 
			post_parent           = int    : 
			guid                  = string : 
			menu_order            = int    : 
			post_type             = string : 投稿情報タイプ（'post'、'page'など）
			post_mime_type        = string : 
			comment_count         = int    : コメント数
			ancestors             = array  : 先祖の投稿IDの配列(先祖の投稿が無い場合は空の配列、配列の最初には親投稿のID、以降その親投稿IDと続き、最後に最上位の投稿IDと順に入る)
			filter                = string : 適用されたフィルター名（'raw'など）

		*/
		//$lposts = get_posts("numberposts=-1&category=" . $lcategory->term_id . "&orderby=id&order=asc");
		$args = array(
			'numberposts' => -1,
			'category'    => (string)$parentCategory,
			'orderby'     => $psort,
			'order'       => $porder
		);
		$lposts = get_posts(http_build_query($args));
		if (count($lposts) == 0) return;

		// インデント設定
		$indent = $this->get_indent($level);

		// 取得投稿記事全件処理
		foreach ($lposts as $lpost) {
			// 所属カテゴリ取得
			$parents = get_the_category($lpost->ID);
			// 指定親カテゴリと直近の所属カテゴリが一致する(直属のカテゴリである)場合のみ出力
			if ($parentCategory == $parents[0]->term_id) {
				// 記事リンクの出力
				echo $indent;
				echo '<li class="list_post">';
				echo '<img id="swfile_' . $lpost->ID . '" src="' . get_template_directory_uri() . '/images/file.gif" alt="投稿記事" />';
				echo '<a href="' . get_permalink($lpost->ID) . '" title="' . $lpost->post_title . '">' . $lpost->post_title . '</a>';
				echo '</li>' . self::CHAR_CR;
			}
		}

	}

	/**
	 * インデント取得
	 *
	 * @access private
	 * @param  int    $count    インデント数
	 * @return string インデント文字列
	 */
	function get_indent($count = 1) {
		return str_pad('', strlen(self::CHAR_INDENT) * $count, self::CHAR_INDENT);
	}

}
?>
